// File stdlib.h for CC1B
#ifndef _STDLIB_H_
#define _STDLIB_H_
//include other libraries here
#include <system/memory.h>
#include <text/character/ctype.h>
#include <text/string/string.h>
#pragma library 1

/**
 * \file stdlib.h
 * \brief Library for string conversion support
 *
 * This library provides functions for string conversions.
 * \n
 */

/**
 * \brief Get absolute value of signed integer
 *
 * Get absolute value of signed integer.
 *
 * \param nbr Number to convert.
 * \return Absolute value of nbr.
 * \n
 */
int abs(int nbr)
{
  if (nbr < 0) return (-nbr);
  return (nbr);
}

/**
 * \brief Convert unsigned decimal string to integer
 *
 * Convert unsigned decimal string to integer.
 *
 * \param decstr Address of unsigned decimal string in ram.
 * \param nbr Address of result integer.
 * \return Number of characters read from decstr, -1 if error.
 * \n
 */
int utoi(char *decstr, int *nbr)
{
  int d,t; d=0;
  *nbr=0;
  while (((*decstr)>='0') && ((*decstr)<='9')) {
    t=*nbr;
	t = 10*t;
	t += *decstr;
	t -= '0';
	decstr++;
    if ((t>=0)&&((*nbr)<0)) return -1;
    d++; *nbr=t;
  }
  return d;
}

/**
 * \brief Convert signed decimal string to integer
 *
 * Convert signed decimal string to integer.
 *
 * \param decstr Address of signed decimal string in ram.
 * \param nbr Address of result integer.
 * \return Number of characters read from decstr, -1 if error.
 * \n
 */
int dtoi(char *decstr, int *nbr)
{
  int len, s;
  if ((*decstr)=='-') {
    s=1;
	++decstr;
  }
  else s=0;
  len = utoi(decstr, nbr);
  if (len<0) return -1;
  if ((*nbr)<0) return -1;
  if (s) {
    *nbr = -*nbr;
	return ++len;
  }
  else return len;
}

/**
 * \brief Convert unsigned octal string to integer
 *
 * Convert unsigned octal string to integer.
 *
 * \param octstr Address of unsigned octal string in ram.
 * \param nbr Address of result integer.
 * \return Number of characters read from octstr, -1 if error.
 * \n
 */
int otoi(char *octstr, int *nbr)
{
  int d,t; d=0;
  *nbr=0;
  while ((*octstr>='0') && (*octstr<='7')) {
    t=*nbr;
    t <<= 3;
	t += *octstr;
	t -= '0';
	octstr++;
    if ((t>=0) && ((*nbr)<0)) return -1;
    d++; *nbr=t;
  }
  return d;
}

/**
 * \brief Convert hexadecimal string to integer
 *
 * Convert hexadecimal string to integer.
 *
 * \param hexstr Address of hexadecimal string in ram.
 * \param nbr Address of result integer.
 * \return Number of characters read from hexstr, -1 if error.
 * \n
 */
int xtoi(char *hexstr, int *nbr)
{
  int d, b;  char *cp; char temp;
  d = *nbr = 0; cp = hexstr;
  while (*cp == '0') ++cp;
  while (1) {
    switch (*cp) {
      case '0': case '1': case '2':
      case '3': case '4': case '5':
      case '6': case '7': case '8':
      case '9':                     b=48; break;
      case 'A': case 'B': case 'C':
      case 'D': case 'E': case 'F': b=55; break;
      case 'a': case 'b': case 'c':
      case 'd': case 'e': case 'f': b=87; break;
      default: return (cp - hexstr);
    }
    if (d < 4) ++d; else return (-1);
    *nbr <<= 4;
	temp = *cp++;
    *nbr += temp;
	*nbr -= b;
  }
}

/**
 * \brief Convert signed decimal string to integer
 *
 * Convert signed decimal string to integer.
 *
 * \param s Address of signed decimal string in ram.
 * \return Signed integer value.
 * \n
 */
int atoi(char *s)
{
  int sgn, n;
  while (isspace(*s)) ++s;
  sgn = 1;
  switch (*s) {
    case '-': sgn = -1;
    case '+': ++s;
  }
  n = 0;
  while (isdigit(*s)) {
    n = 10 * n;
    n += (*s);
    n -= '0';
	s++;
  }
  n *= sgn;
  return (n);
}

/**
 * \brief Convert unsigned decimal string to integer for base
 *
 * Convert signed decimal string to integer using base b.\n
 * This is a non-standard function.
 *
 * \param s Address of unsigned decimal string in ram.
 * \param b Base for conversion: (2=binary,8=octal,10=decimal,16=hexadecimal).
 * \return Signed integer value.
 * \n
 */
int atoib(char *s, int b)
{
  int n; char digit;
  n = 0;
  while (isspace(*s)) ++s;
  digit = (127 & *s);
  while (digit >= '0') {
    if (digit >= 'a')      digit -= 87;
    else if (digit >= 'A') digit -= 55;
    else                   digit -= '0';
    if (digit >= b) break;
    n *= b;
	n += digit;
    s++;
    digit = (127 & *s);
  }
  return (n);
}

/**
 * \brief Convert integer to signed decimal string
 *
 * Convert integer to signed decimal string.
 *
 * \param nbr Signed value.
 * \param str Address of character array in ram.
 * \param sz Option,\n
 * sz > 0: terminate with null byte, sz = 0: find end of string, sz < 0: use last byte for data.
 * \return str.
 * \n
 */
char *itod(int nbr, char *str, int sz)
{
  char sgn; char temp;
  if (nbr<0) {nbr = -nbr; sgn='-';}
  else sgn=' ';
  if (sz>0) str[--sz]=0;
  else if (sz<0) sz = -sz;
       else while (str[sz]!=0) ++sz;
  while(sz) {
    temp = (char)nbr % 10;
	str[--sz] = temp;
	str[sz] += '0';
	nbr = (char)nbr / 10;
    if (nbr==0) break;
  }
  if (sz) str[--sz]=sgn;
  while (sz>0) str[--sz]=' ';
  return str;
}

/**
 * \brief Convert integer to octal string
 *
 * Convert integer to octal string.
 *
 * \param nbr Signed value.
 * \param str Address of character array in ram.
 * \param sz Option,\n
 * sz > 0: terminate with null byte, sz = 0: find end of string, sz < 0: use last byte for data.
 * \return str.
 * \n
 */
char *itoo(int nbr, char *str, int sz)
{
  int digit;
  if (sz>0) str[--sz]=0;
  else if(sz<0) sz = -sz;
       else while (str[sz]!=0) ++sz;
  while(sz) {
    digit = nbr &7;
    nbr = ((char)nbr>>3);
	nbr &= 8191;
    str[--sz]=digit+48;
    if (nbr==0) break;
  }
  while (sz) str[--sz]=' ';
  return str;
}

/**
 * \brief Convert integer to unsigned decimal string
 *
 * Convert integer to unsigned decimal string.
 *
 * \param nbr Unsigned value.
 * \param str Address of character array in ram.
 * \param sz Option,\n
 * sz > 0: terminate with null byte, sz = 0: find end of string, sz < 0: use last byte for data.
 * \return str.
 * \n
 */
char *itou(int nbr, char *str, int sz)
{
  int lowbit; char temp;
  if (sz>0) str[--sz]=0;
  else if (sz<0) sz = -sz;
       else while (str[sz]!=0) ++sz;
  while(sz) {
    lowbit=nbr&1;
    nbr=(nbr>>1)&32767;  /* divide by 2 */
    temp = (char)nbr % 5;
	str[--sz] = temp;
	str[sz] <<= 1;
	str[sz] += lowbit;
	str[sz] += '0';
	nbr = (char)nbr / 5;
    if (nbr==0) break;
  }
  while (sz) str[--sz]=' ';
  return str;
}

/**
 * \brief Convert integer to hexadecimal string
 *
 * Convert integer to hexadecimal string.
 *
 * \param nbr Unsigned value.
 * \param str Address of character array in ram.
 * \param sz Option,\n
 * sz > 0: terminate with null byte, sz = 0: find end of string, sz < 0: use last byte for data.
 * \return str.
 * \n
 */
char *itox(int nbr, char *str, int sz)
{
  int digit, offset;
  if (sz>0) str[--sz]=0;
  else if(sz<0) sz = -sz;
       else while (str[sz]!=0) ++sz;
  while(sz) {
    digit = nbr & 15;
	nbr = (char)nbr>>4;
	nbr &= 4095;
    if (digit<10) offset=48; else offset=55;
    str[--sz] = digit+offset;
    if (nbr==0) break;
  }
  while (sz) str[--sz]=' ';
  return str;
}

/**
 * \brief Convert integer to signed decimal string
 *
 * Convert integer to signed decimal string.
 *
 * \param n Signed value.
 * \param s Address of character array in ram.
 * \return str.
 * \n
 */
char *itoa(int n, char *s)
{
  int sgn;
  char *ptr;
  ptr = s;
  sgn = n;
  if (sgn < 0) n = -n;
  do {
    _MainTemp = (char)n % 10;
	*ptr = _MainTemp;
    *ptr += '0';
	ptr++;
	n = (char)n/10;
  } while (n > 0);
  if (sgn < 0) *ptr++ = '-';
  *ptr = '\0';
  return reverse(s);
}

/**
 * \brief Convert integer to unsigned decimal string for base
 *
 * Convert integer to unsigned decimal string for base b.
 *
 * \param n Unsigned value.
 * \param s Address of character array in ram.
 * \param b Base (2=binary, 8=octal, 10=decimal, 16=hexadecimal),
  * \return str.
 * \n
 */
char *itoab(int n, char *s, int b)
{
  char *ptr; char temp;
  int lowbit;
  ptr = s;
  b >>= 1;
  do {
    lowbit = n & 1;
    n = (n >> 1) & 32767;
    temp = (char)n % (char)b;
	*ptr = temp;
    *ptr <<= 1;
    *ptr += lowbit;
    if(*ptr < 10) *ptr += '0'; else *ptr += 55;
    ++ptr;
	n = (char)n / (char)b;
  } while(n != 0);
  *ptr = 0;
  return reverse(s);
}

/**
 * \brief Laft adjust and null-terminate a string
 *
 * Left adjust and null-terminate a string.
 *
 * \param str String to adjust.
 * \return str.
 * \n
 */
char *left(char *str)
{
  char *str2; char temp;
  str2=str;
  while (*str2==' ') ++str2;
  while (*str2) {
    temp = *str2;
    *str = temp;
    str++;
    str2++;
  }
  *str = 0;
  return str;
}

/**
 * \brief Place n occurences of ch at dest
 *
 * Place n occurrences of ch at dest.
 *
 * \param dest String in which to place ch.
 * \param ch Character to plce in dest.
 * \param n Number of times to place ch in dest.
 * \return dest.
 * \n
 */
char *pad(char *dest, char ch, int n)
{
  while (n) {
    n--;
	*dest++ = ch;
  }
  return dest;
}

/**
 * \brief Reverse string in place
 *
 * Reverse string in place.
 *
 * \param s String to reverse.
* \return s.
* \n
 */
char *reverse(char *s)
{
  char *j;
  int c;
  j = s + strlen(s);
  j--;
  while (s < j) {
    c = *s;
	_MainTemp = *j;
    *s++ = _MainTemp;
	_MainTemp = c;
    *j-- = _MainTemp;
  }
  return s;
}

/**
 * \brief Get sign of integer
 *
 * Get sign of integer.
 *
 * \param nbr Signed integer.
 * \return -1, 0, +1 depending on the sign of nbr.
 * \n
 */
int sign(int nbr)
{
  if (nbr > 0)  return 1;
  if (nbr == 0) return 0;
  return -1;
}

#pragma library 0
#endif
// End of file stdlib.h
